@@ -2,11 +2,11 @@
  * Implement the core of the --chmod option.
  *
  * Copyright (C) 2002 Scott Howard
- * Copyright (C) 2005, 2006 Wayne Davison
+ * Copyright (C) 2005-2014 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -15,11 +15,11 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ * with this program; if not, visit the http://fsf.org website.
  */
 
 #include "rsync.h"
+#include "itypes.h"
 
 extern mode_t orig_umask;
 
@@ -36,10 +36,12 @@ struct chmod_mode_struct {
 #define CHMOD_ADD 1
 #define CHMOD_SUB 2
 #define CHMOD_EQ  3
+#define CHMOD_SET 4
 
 #define STATE_ERROR 0
 #define STATE_1ST_HALF 1
 #define STATE_2ND_HALF 2
+#define STATE_OCTAL_NUM 3
 
 /* Parse a chmod-style argument, and break it down into one or more AND/OR
  * pairs in a linked list.  We return a pointer to new items on succcess
@@ -88,6 +90,10 @@ struct chmod_mode_struct *parse_chmod(const char *modestr,
 				curr_mode->ModeAND = CHMOD_BITS - (where * 7) - (topoct ? topbits : 0);
 				curr_mode->ModeOR  = bits + topoct;
 				break;
+			case CHMOD_SET:
+				curr_mode->ModeAND = 0;
+				curr_mode->ModeOR  = bits;
+				break;
 			}
 
 			curr_mode->flags = flags;
@@ -100,17 +106,18 @@ struct chmod_mode_struct *parse_chmod(const char *modestr,
 			where = what = op = topoct = topbits = flags = 0;
 		}
 
-		if (state != STATE_2ND_HALF) {
+		switch (state) {
+		case STATE_1ST_HALF:
 			switch (*modestr) {
 			case 'D':
 				if (flags & FLAG_FILES_ONLY)
 					state = STATE_ERROR;
-			  	flags |= FLAG_DIRS_ONLY;
+				flags |= FLAG_DIRS_ONLY;
 				break;
 			case 'F':
 				if (flags & FLAG_DIRS_ONLY)
 					state = STATE_ERROR;
-			  	flags |= FLAG_FILES_ONLY;
+				flags |= FLAG_FILES_ONLY;
 				break;
 			case 'u':
 				where |= 0100;
@@ -139,10 +146,17 @@ struct chmod_mode_struct *parse_chmod(const char *modestr,
 				state = STATE_2ND_HALF;
 				break;
 			default:
-				state = STATE_ERROR;
+				if (isDigit(modestr) && *modestr < '8' && !where) {
+					op = CHMOD_SET;
+					state =  STATE_OCTAL_NUM;
+					where = 1;
+					what = *modestr - '0';
+				} else
+					state = STATE_ERROR;
 				break;
 			}
-		} else {
+			break;
+		case STATE_2ND_HALF:
 			switch (*modestr) {
 			case 'r':
 				what |= 4;
@@ -151,7 +165,7 @@ struct chmod_mode_struct *parse_chmod(const char *modestr,
 				what |= 2;
 				break;
 			case 'X':
-			  	flags |= FLAG_X_KEEP;
+				flags |= FLAG_X_KEEP;
 				/* FALL THROUGH */
 			case 'x':
 				what |= 1;
@@ -169,6 +183,15 @@ struct chmod_mode_struct *parse_chmod(const char *modestr,
 				state = STATE_ERROR;
 				break;
 			}
+			break;
+		case STATE_OCTAL_NUM:
+			if (isDigit(modestr) && *modestr < '8') {
+				what = what*8 + *modestr - '0';
+				if (what > CHMOD_BITS)
+					state = STATE_ERROR;
+			} else
+				state = STATE_ERROR;
+			break;
 		}
 		modestr++;
 	}
