@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1996 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
- * Copyright (C) 2004-2013 Wayne Davison
+ * Copyright (C) 2004-2014 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -473,12 +473,15 @@ void parse_name_map(char *map, BOOL usernames)
 					usernames ? "user" : "group", cp);
 				exit_cleanup(RERR_SYNTAX);
 			}
-			if (dash)
+			if (dash) {
+				*dash = '\0';
 				noiu.max_id = id_parse(dash+1);
-			else
+			} else
 				noiu.max_id = 0;
 			flags = 0;
 			id1 = id_parse(cp);
+			if (dash)
+				*dash = '-';
 		} else if (strpbrk(cp, "*[?")) {
 			flags = NFLAGS_WILD_NAME_MATCH;
 			noiu.name = cp;
